#!/usr/bin/perl -w                                         # -*- perl -*-

use strict;
use Benchmark;
use lib qw( /home/abw/tt2/lib /home/abw/tt3/lib );
use Template;
use Template::Stash;
use Template::Stash::v3;

local $/ = undef;
my $DEBUG  = 0;
my $ITER   = 1000;
my $source = 'test/stash';

my @days = qw( Monday Tuesday Wednesday Thursday Friday Saturday Sunday );
my ($a, $b, $c, $d, $e, $f, $g, $h, $i, $j, $k, $l, $m, 
    $n, $o, $p, $q, $r, $s, $t, $u, $v, $w, $x, $y, $z) = 
	qw( alpha bravo charlie delta echo foxtrot golf hotel india 
	    juliet kilo lima mike november oscar papa quebec romeo 
	    sierra tango umbrella victor whisky x-ray yankee zulu );

my $vars = { 
    'a' => $a,
    'b' => $b,
    'c' => $c,
    'd' => $d,
    'e' => $e,
    'f' => {
	'g' => $g,
	'h' => $h,
	'i' => {
	    'j' => $j,
	    'k' => $k,
	    'l' => sub { return {
		m => [
		      { n => $n, o => $o },
		      { p => $p, q => $q },
		      ],
		n => { r => $r },
		} },
	},
    },
    loop => 10,
    days => \@days,
};

my $stash_normal  = Template::Stash->new($vars);
my $stash_v3      = Template::Stash::v3->new($vars);

my $tt_normal  = Template->new( STASH => $stash_normal );
my $tt_v3      = Template->new( STASH => $stash_v3 );

sub normal {
    my $src = $source;
    my $out = '';
    $tt_normal->process($src, undef, \$out)
	|| die $tt_normal->error();
    print "normal completed\n$out\n\n"
	if $DEBUG;
}

sub v3 {
    my $src = $source;
    my $out = '';
    $tt_v3->process($src, undef, \$out)
	|| die $tt_v3->error();
    print "v3 completed\n$out\n\n"
	if $DEBUG;
}


timethese($DEBUG ? 1 : $ITER, {
    normal  => \&normal,
    v3      => \&v3,
});

