#============================================================= -*-perl-*-
#
# Template::Facility
#
# DESCRIPTION
#   Object class to represent a reusable facility.
#
# AUTHOR
#   Andy Wardley   <abw@kfs.org>
#
# COPYRIGHT
#   Copyright (C) 2001 Andy Wardley.  All Rights Reserved.
#
#   This module is free software; you can redistribute it and/or
#   modify it under the same terms as Perl itself.
#
# REVISION
#   $Id: Facility.pm,v 1.2 2001/06/19 16:15:25 abw Exp $
#
#========================================================================

package Template::Facility;

require 5.005;

use strict;
use Template::Base;
use vars qw( $VERSION $DEBUG $ERROR $ETYPE $FEATURES );
use base qw( Template::Base );

$VERSION = sprintf("%d.%02d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/);
$DEBUG   = 0 unless defined $DEBUG;
$ERROR   = '';
$ETYPE   = 'facility';

$FEATURES = {
    # stuff to go here...
};


#------------------------------------------------------------------------
# acquire($context)
#
# Binds the facility to a Context.
#------------------------------------------------------------------------

sub acquire {
    my $self = shift;
    my $context = shift
	|| return $self->error('no context to acquire');

    return $self->error('already acquired by a context')
	if $self->{ _CONTEXT };

    $self->{ _CONTEXT } = $context;

    $self->DEBUG("$context acquired $self\n");

    return $self->attach($context);
}


#------------------------------------------------------------------------
# attach($context)
#
# Attach features to a Context object.
#------------------------------------------------------------------------

sub attach {
    my $self = shift;
    my $context = shift 
	|| $self->{ _CONTEXT }
	|| return $self->error('no context to attach to');

    $self->DEBUG("$self attaching to $context\n");

    return 1;
}


#------------------------------------------------------------------------
# detach($context)
#
# Detach features from a Context object.
#------------------------------------------------------------------------

sub detach {
    my $self = shift;
    my $context = shift 
	|| $self->{ _CONTEXT }
	|| return $self->error('no context to detach from');

    $self->DEBUG("$self detaching from $context\n");

    return 1;
}
    

#------------------------------------------------------------------------
# release()
#
# Release the facility from a Context previously bound to by acquire().
#------------------------------------------------------------------------

sub release {
    my $self = shift;
    my $context = $self->{ _CONTEXT }
        || return $self->error('no context to release');

    $self->DEBUG("$context releasing $self\n");

    my $result = $self->detach($context);
    undef $self->{ _CONTEXT };

    return $result;
}


sub context {
    my $self = shift;
    return $self->{ _CONTEXT } 
        || $self->error("no context");
}


sub DESTROY {
    my $self = shift;
    $self->release() if $self->{ _CONTEXT };
}

1;

__END__
