#============================================================= -*-perl-*-
#
# t/base.t
#
# Test the Template::Base.pm module.
#
# Written by Andy Wardley <abw@kfs.org>
#
# This is free software; you can redistribute it and/or modify it
# under the same terms as Perl itself.
#
# $Id: base.t,v 1.3 2001/06/23 09:30:02 abw Exp $
#
#========================================================================

$^W = 1;

use strict;
use lib qw( ./lib ../lib );
use Template::Test;
use Template::Base;

#Template::Base::DEBUG = 1;

### NOTE: don't forget to update this value or comment it out ###
ntests(103);

my ($pkg, $mod);


#------------------------------------------------------------------------
# Template::Test::Fail always fails, but we check it reports errors OK

package Template::Test::Fail;
use base qw( Template::Base );
use vars qw( $ERROR );

sub init {
    my $self = shift;
    return $self->error('expected failure');
}

package main;

# instantiate a base class object and test error reporting/returning
$mod = Template::Base->new();
ok( $mod );
ok( ! defined $mod->error('barf') );
ok( $mod->error() eq 'barf' );

# Template::Test::Fail should never work, but we check it reports errors OK
$pkg = 'Template::Test::Fail';
ok( ! $pkg->new() );
match( $pkg->error, 'expected failure' );
match( $Template::Test::Fail::ERROR, 'expected failure' );


#------------------------------------------------------------------------
# Template::Test::Name should only work with a 'name'parameters

package Template::Test::Name;
use base qw( Template::Base );
use vars qw( $ERROR );

sub init {
    my ($self, $params) = @_;
    $self->{ NAME } = $params->{ name } 
	|| return $self->error("No name!");
    return $self;
}

sub name {
    $_[0]->{ NAME };
}

package main;

$mod = Template::Test::Name->new();
ok( ! $mod );
match( $Template::Test::Name::ERROR, 'No name!' );
match( Template::Test::Name->error(), 'No name!' );

# give it what it wants...
$mod = Template::Test::Name->new({ name => 'foo' });
ok( $mod );
ok( ! $mod->error() );
match( $mod->name(), 'foo' );

# ... in 2 different flavours
$mod = Template::Test::Name->new(name => 'foo');
ok( $mod );
ok( ! $mod->error() );
match( $mod->name(), 'foo' );


#------------------------------------------------------------------------
# Template::Test::Args expects some @ARGUMENTS

package Template::Test::Args;
use base qw( Template::Base );
use vars qw( $ERROR @ARGS );

@ARGS = qw( name );



