#!/usr/bin/perl -w                                         # -*- perl -*-

use strict;
use lib qw( ../lib ./lib ./t . );
use Template::Context;
use Template::Facility;
use Template::Test;
use vars qw( $MSG );

require 'myobjs.pl';

#$Template::Context::DEBUG = 1;
#$Template::Facility::DEBUG = 1;

local $" = ', ';

#------------------------------------------------------------------------
package main;

my $context = Template::Context->new()
    || die Template::Context->error();

assert( $context );

my $foo = My::Facility->new()
    || die My::Facility->error();
assert( $foo );

my $bar = My::Facility->new( name => 'The Bar Facility')
    || die My::Facility->error();
assert( $bar );

ok( $context->acquire(foo => $foo) );
match( $MSG, "$foo->acquire($context)" );

my $f = $context->facility('none');
ok( ! $f );
match( $context->error(), "none: no such facility" );

$f = $context->facility('foo');
ok( $f );
match( $f, $foo );

ok( $context->release('foo') );
match( $MSG, "$foo->release($context)" );

$f = $context->facility('foo');
ok( ! $f );
match( $context->error(), "foo: no such facility" );

#--

ok( $context->acquire(foo => $foo) );
match( $MSG, "$foo->acquire($context)" );

ok( $context->acquire(bar => $bar) );
match( $MSG, "$bar->acquire($context)" );

match( $context->facility('foo'), $foo );
match( $context->facility('bar'), $bar );

my ($f1, $f2) = $context->facilities(qw( foo bar ));
match( $f1, $foo );
match( $f2, $bar );

my $f12 = $context->facilities(qw( foo bar ));
ok( $f12 );
match( $f12->[0], $foo );
match( $f12->[1], $bar );

my $facs = $context->facilities();
match( $facs->{ foo }, $foo );
match( $facs->{ bar }, $bar );

#--

my $observer = My::Observer->new();

ok( $context->observe( foo => $observer, hello => 3.14, 2.718 ) );
ok( $context->notify('foo', 1.414) );
match( $MSG, "$observer->hello(3.14, 2.718, 1.414)" );

$MSG = '';

ok( $context->observe( bar => $observer, 'world' ) );
ok( $context->notify('bar', 99 ) );
match( $MSG, "$observer->world(99)" );

ok( $context->observe( foo => $observer, 'world', 123 ) );
ok( $context->notify('foo', 99 ) );
match( $MSG, "$observer->hello(3.14, 2.718, 99)$observer->world(123, 99)" );
