#!/usr/bin/perl -w                                         # -*- perl -*-
#
# NOTE: some of these tests may rely on the particular hashing 
# order used by Perl on your system.
#
# NOTE: this tests v3 stash in a v2 framework so make sure that it finds
# V2 file *before* any of the v3 files.

use strict;
use lib qw( /home/abw/tt2/lib ../lib ./lib );
use Template;
use Template::Test;
use Template::Stash::v3;

local $/ = undef;
my $DEBUG  = 0;


#------------------------------------------------------------------------
package Foo;
#------------------------------------------------------------------------

sub new {
    my $class = shift;
    bless [ @_ ], $class;
}

sub TT_REPEAT {
    my ($self, $n) = @_;
    return [ @$self x $n ];
}

sub TT_LIST {
    my $self = shift;
    return [ @$self ];
}

sub TT_ITEM {
    my $self = shift;
    return "I am Foo [@$self]";
}

#------------------------------------------------------------------------
package main;
#------------------------------------------------------------------------

my @days = qw( Mon Tue Wed Thu Fri Sat Sun );
my $vars = { 
    hash  => {
        zero   => 0,
	one    => 1,
	two    => 2,
        three  => 3,
    },
    today => 'Tuesday',
    days  => \@days,
    foo   => Foo->new( 2, 3, 5, 7, 11 ),
    debug => sub { $Template::Stash::v3::DEBUG = shift },
    warn  => sub { $Template::Stash::v3::WARNINGS = shift },
};

my $stash_v3 = Template::Stash::v3->new($vars);
my $tt_v3    = Template->new( STASH => $stash_v3 );

test_expect(\*DATA, $tt_v3);

__END__
-- test --
[% days.REPEAT.JOIN %]
-- expect --
Mon, Tue, Wed, Thu, Fri, Sat, Sun

-- test --
[% days.REPEAT(2).JOIN %]
-- expect --
Mon, Tue, Wed, Thu, Fri, Sat, Sun, Mon, Tue, Wed, Thu, Fri, Sat, Sun

-- test --
[% today.REPEAT %]
-- expect --
Tuesday

-- test --
[% today.REPEAT(2) %]
-- expect --
TuesdayTuesday

-- test --
[% today.LIST.REPEAT(2).JOIN %]
-- expect --
Tuesday, Tuesday

-- test --
[% hash.REPEAT(2).JOIN %]
-- expect --
one, 1, zero, 0, three, 3, two, 2, one, 1, zero, 0, three, 3, two, 2

