#!/usr/bin/perl -w                                         # -*- perl -*-

use strict;
use lib qw( ../lib ./lib ./t . );
use Template::Service;
use Template::Context;
use Template::Facility;
use Template::Test;

ntests(15);

my $DEBUG = 0;
$Template::Service::DEBUG = $DEBUG;
$Template::Context::DEBUG = $DEBUG;

require 'myobjs.pl';

my $c1 = Template::Context->new()
    || die Template::Context->error();

my $c2 = Template::Context->new()
    || die Template::Context->error();

my $c3 = Template::Context->new()
    || die Template::Context->error();


my $service = Template::Service->new({
    CONTEXTS => {
	alpha => $c1,
	beta  => $c2,
	gamma => $c3,
    },
});

assert( $service, $Template::Service::ERROR );

match( $service->context('alpha'), $c1 );
match( $service->context('beta'),  $c2 );
match( $service->context('gamma'), $c3 );

match( $service->context(), $c3, $service->error() );

$service = Template::Service->new({
    CONTEXTS => {
	alpha   => $c1,
	beta    => $c2,
	gamma   => $c3,
	default => $c2,
    },
});

assert( $service, Template::Service->error() );
match( $service->context(), $c2 );

$c1 = Template::Context->new();
assert( $c1, $Template::Context::ERROR );
$c2 = Template::Context->new();
assert( $c1, $Template::Context::ERROR );
$c3 = Template::Context->new();
assert( $c1, $Template::Context::ERROR );

$service = Template::Service->new({
    CONTEXTS => {
	alpha   => $c1,
	beta    => $c2,
	gamma   => $c3,
	default => 'alpha',
    },
});

assert( $service, Template::Service->error() );
match( $service->context(), $c1 );

my $current = $service->context();
match( $current, $c1 );

$current = $current->switch('beta');
match( $current, $c2 );

$current = $current->switch('alpha');
match( $current, $c1 );

