#!/usr/bin/perl -w                                         # -*- perl -*-
#
# NOTE: some of these tests may rely on the particular hashing 
# order used by Perl on your system.
#
# NOTE: this tests v3 stash in a v2 framework so make sure that it finds
# V2 file *before* any of the v3 files.

use strict;
use lib qw( /home/abw/tt2/lib ../lib ./lib );
use Template;
use Template::Test;
use Template::Stash::v3;

local $/ = undef;
my $DEBUG  = 0;


#------------------------------------------------------------------------
package Foo;
#------------------------------------------------------------------------

sub new {
    my $class = shift;
    bless [ @_ ], $class;
}

sub TT_SORT {
    my $self = shift;
    my (@numbers, @words);
    foreach (@$self) {
	push(@numbers, $_), next if /^[+-]?\d+(.\d+)?/;
	push(@words, $_);
    }
    return [ numbers => ( sort { $a <=> $b } @numbers ),
	     words   => ( sort { lc $a cmp lc $b } @words   ) ];
}

sub TT_LIST {
    my $self = shift;
    return [ @$self ];
}

sub TT_ITEM {
    my $self = shift;
    return "I am Foo [@$self]";
}

#------------------------------------------------------------------------
package main;
#------------------------------------------------------------------------

my @days = qw( Monday Tuesday Wednesday Thursday Friday Saturday Sunday );
my @ids  = qw( ABC01 add02 BOO bar 1foo 2bar 10wiz 11waz );
my $vars = { 
    hash  => {
        zero   => 0,
	one    => 1,
	two    => 2,
        three  => 3,
        four   => 4,
        five   => 5,
	ten    => 10,
	twenty => 20,
    },
    days  => \@days,
    foo   => Foo->new( 0, 1, -1, 2, 10, 3.14, 2.718, 'epsilon', 'G', 'E', 'c', ),
    debug => sub { $Template::Stash::v3::DEBUG = shift },
    warn  => sub { $Template::Stash::v3::WARNINGS = shift },
};

my $stash_v3 = Template::Stash::v3->new($vars);
my $tt_v3    = Template->new( STASH => $stash_v3 );

test_expect(\*DATA, $tt_v3);

__END__
-- test --
[% days.JOIN %]
-- expect --
Monday, Tuesday, Wednesday, Thursday, Friday, Saturday, Sunday

-- start --
-- test --
[% CALL warn(1); days.SORT.JOIN %]
-- expect --
Friday, Monday, Saturday, Sunday, Thursday, Tuesday, Wednesday

-- test --
[% days.SORT(order => 'alpha').JOIN %]
-- expect --
Friday, Monday, Saturday, Sunday, Thursday, Tuesday, Wednesday

-- test --
[% days.SORT(order => 'number').JOIN %]
-- expect --
Monday, Tuesday, Wednesday, Thursday, Friday, Saturday, Sunday

-- test --
[% foo.SORT.JOIN %]
-- expect --
numbers, -1, 0, 1, 2, 2.718, 3.14, 10, words, c, E, epsilon, G

-- test --
[% foo.LIST.SORT.JOIN %]
-- expect --
-1, 0, 1, 10, 2, 2.718, 3.14, c, E, epsilon, G

-- test --
[% foo.LIST.SORT( order => 'number' ).JOIN %]
-- expect --
-1, 0, c, epsilon, G, E, 1, 2, 2.718, 3.14, 10

-- test --
[% foo.LIST.SORT( order => 'number' ).REVERSE.JOIN %]
-- expect --
10, 3.14, 2.718, 2, 1, E, G, epsilon, c, 0, -1

-- test --
[% hash.SORT.JOIN %]
-- expect --
five, 5, four, 4, one, 1, ten, 10, three, 3, twenty, 20, two, 2, zero, 0

-- test --
[% hash.SORT( item => 'keys' ).JOIN %]
-- expect --
five, 5, four, 4, one, 1, ten, 10, three, 3, twenty, 20, two, 2, zero, 0

-- test --
[% hash.SORT( item => 'values' ).JOIN %]
-- expect --
zero, 0, one, 1, ten, 10, two, 2, twenty, 20, three, 3, four, 4, five, 5

-- test --
[% hash.SORT( item => 'values', order => 'number' ).JOIN %]
-- expect --
zero, 0, one, 1, two, 2, three, 3, four, 4, five, 5, ten, 10, twenty, 20

-- test --
[% hash.JOIN( item => 'values', order => 'number' ) %]
-- expect --
zero = 0, one = 1, two = 2, three = 3, four = 4, five = 5, ten = 10, twenty = 20

-- test --
hash = {
    [% hash.JOIN( ",\n    ", item => 'values', order => 'number' ) %],
}
-- expect --
hash = {
    zero = 0,
    one = 1,
    two = 2,
    three = 3,
    four = 4,
    five = 5,
    ten = 10,
    twenty = 20,
}


-- test --
my $hash = {
    [% hash.JOIN( order  = 'number'
	          item   = 'values'
                  joint  = ",\n    "
                  equals = ' => ' ) 
    %],
};
-- expect --
my $hash = {
    zero => 0,
    one => 1,
    two => 2,
    three => 3,
    four => 4,
    five => 5,
    ten => 10,
    twenty => 20,
};

-- test --
hash = { [% hash.JOIN(', ', ' => ', item => 'values', order => 'number' ) %] }
-- expect --
hash = { zero => 0, one => 1, two => 2, three => 3, four => 4, five => 5, ten => 10, twenty => 20 }

-- test --
hash = { [% hash.JOIN(',', equals = '=>', item = 'values', order = 'number' ) %] }
-- expect --
hash = { zero=>0,one=>1,two=>2,three=>3,four=>4,five=>5,ten=>10,twenty=>20 }

-- test --
hash = { [% hash.JOIN(joint => ' + ', equals => ' => ', item => 'values', order => 'number' ) %] }
-- expect --
hash = { zero => 0 + one => 1 + two => 2 + three => 3 + four => 4 + five => 5 + ten => 10 + twenty => 20 }

