#============================================================= -*-perl-*-
#
# t/stash00.t
#
# Test the Template::Stash::v3 module via Perl calls.
#
# Written by Andy Wardley <abw@kfs.org>
#
# Copyright (C) 1996-2001 Andy Wardley.  All Rights Reserved.
# Copyright (C) 1998-2001 Canon Research Centre Europe Ltd.
#
# This is free software; you can redistribute it and/or modify it
# under the same terms as Perl itself.
#
# $Id: stash00.t,v 1.1 2001/06/27 13:37:48 abw Exp $
#
#========================================================================

use strict;
use lib qw( ../lib );
use Template::Stash::v3;
use Template::Test;
$^W = 1;
local $" = ', ';

ntests(124);

#$Template::Stash::v3::DEBUG = 1;


#------------------------------------------------------------------------
package Foo;
#------------------------------------------------------------------------

sub new {
    my $class = shift;
    bless { foo => 'bar', wiz => 'waz' }, $class;
}

sub bar {
    return 'Foo::Bar';
};

sub plonk {
    my $self = shift;
    return "Plonk! [@_]";
}

sub TT_ITEM {
    my $self = shift;
    return "Foo->TT_ITEM(@_)";
}


#------------------------------------------------------------------------
package Bar;
#------------------------------------------------------------------------

use base qw( Foo );
use vars qw( $AUTOLOAD );

sub TT_REF {
    my $self = shift;
    return "Bar->TT_REF(@_)";
}
sub TT_LIST {
    my $self = shift;
    return "Bar->TT_LIST(@_)";
}
sub TT_HASH {
    my $self = shift;
    return "Bar->TT_HASH(@_)";
}
sub TT_LENGTH {
    my $self = shift;
    return "Bar->TT_LENGTH(@_)";
}
sub TT_SIZE {
    my $self = shift;
    return "Bar->TT_SIZE(@_)";
}

sub TT_MAX {
    my $self = shift;
    return "Bar->TT_MAX(@_)";
}

sub TT_FIRST {
    my $self = shift;
    return "Bar->TT_FIRST(@_)";
}

sub TT_LAST {
    my $self = shift;
    return "Bar->TT_LAST(@_)";
}

sub TT_CAN {
    my ($self, $method) = @_;
    return $method if ($method =~ /^plank|plunk$/);
}

sub AUTOLOAD {
    my $self = shift;
    my $method = $AUTOLOAD;
    $method =~ s/.*:://;
    return if $method eq 'DESTROY';
    return "Bar->$method(@_)";
}
    

#------------------------------------------------------------------------
package main;
#------------------------------------------------------------------------
    
my $count = 20;
my $data = {
    foo => 10,
    bar => {
	baz => 20,
	boz => 30,
    },
    baz => sub {
	return {
	    boz => ($count += 10),
	    biz => (shift || '<undef>'),
	};
    },
    wiz => sub {
	return "wiz(" . join(', ', @_) . ")";
    },
    obj   => Foo->new(),
    objx  => Bar->new(),
    items => [ 1, 2, 3 ],
};

my $stash = Template::Stash::v3->new($data);

goto START;
START:

#------------------------------------------------------------------------
# basic access
#------------------------------------------------------------------------

my $result = $stash->operate(['foo', 0]);
match( $result, 10 );

$result = $stash->operate(['bar', 0, 'baz', 0]);
match( $result, 20 );

$result = $stash->operate(['baz', 0, 'boz', 0]);
match( $result, 30 );

$result = $stash->operate(['obj', 0, 'bar', [23]]);
match( $result, 'Foo::Bar' );

$result = $stash->operate(['non', 0]);
ok( ! $result );

$result = $stash->operate(['non', 0, 'existant', 0]);
ok( ! $result );

$result = $stash->operate(['items', 0, '0', 0]);
match( $result, 1 );

$result = $stash->operate(['items', 0, 'barf', 0]);
ok( ! $result, 'no result?' );

$result = $stash->operate(['bar', 0]);
ok( $result );


#------------------------------------------------------------------------
# REF
#------------------------------------------------------------------------


match( $stash->operate(['foo', 0, 'REF', 0]), '' );
match( $stash->operate(['bar', 0, 'REF', 0]), 'HASH' );
match( $stash->operate(['baz', 0, 'REF', 0]), 'CODE' );
match( $stash->operate(['obj', 0, 'REF', '0']), 'Foo' );
match( $stash->operate(['objx', 0, 'REF', [1]]), 'Bar->TT_REF(1)' );

#------------------------------------------------------------------------
# ITEM
#------------------------------------------------------------------------

$result = $stash->operate(['foo', 0, 'ITEM', 0]);
match( $result, 10 );

$result = $stash->operate(['bar', 0, 'ITEM', ['baz']]);
match( $result, 20 );

$result = $stash->operate(['bar', 0, 'ITEM', ['plonk']]);
ok( ! $result );

$result = $stash->operate(['bar', 0], 'ITEM', 0);
ok( $result );

$result = $stash->operate(['bar', 0, 'ITEM', []]);
ok( $result );

$result = $stash->operate(['bar', 0, 'ITEM', [0]]);
ok( ! $result );

$result = $stash->operate(['obj', 0, 'ITEM', 0]);
match( $result, 'Foo->TT_ITEM()' );

$result = $stash->operate(['obj', 0, 'ITEM', []]);
match( $result, 'Foo->TT_ITEM()' );

$result = $stash->operate(['obj', 0, 'ITEM', [1]]);
match( $result, 'Foo->TT_ITEM(1)' );

$result = $stash->operate(['obj', 0, 'ITEM', [1, 2, 3]]);
match( $result, 'Foo->TT_ITEM(1, 2, 3)' );

$result = $stash->operate(['items', 0, 1, 0]);
match( $result, '2' );

$result = $stash->operate(['items', 0, 'ITEM', 0, 1, 0]);
match( $result, '2' );

$result = $stash->operate(['items', 0, 'ITEM', [0]]);
match( $result, '1' );

$result = $stash->operate(['items', 0, 'ITEM', [1]]);
match( $result, '2' );

match( $stash->operate(['objx', 0, 'ITEM', [1]]),
       'Foo->TT_ITEM(1)' );


#------------------------------------------------------------------------
# LIST
#------------------------------------------------------------------------

$result = $stash->operate(['foo', 0, 'LIST', 0]);
ok( $result );
match($result->[0], 10);

$result = $stash->operate(['items', 0, 'LIST', 0]);
ok( $result );
match($result->[0], 1);

$result = $stash->operate(['items', 0, 'LIST', [2]]);
ok( $result );
match( $result->[1], 2 );

$result = $stash->operate(['bar', 0, 'LIST', 0]);
ok( $result );
$result = [ sort @$result ];
match( $result->[0], 20 );
match( $result->[1], 30 );
match( $result->[2], 'baz' );
match( $result->[3], 'boz' );

$result = $stash->operate(['bar', 0, 'LIST', ['hashrefs']]);
ok( $result );
match( $result->[0]->{ key }, 'baz' );
match( $result->[0]->{ value }, 20 );
match( $result->[1]->{ key }, 'boz' );
match( $result->[1]->{ value }, 30 );

my $keys = $stash->operate(['bar', 0, 'LIST', ['keys']]);
ok( $keys );
match( join(', ', sort @$keys), 'baz, boz' );

my $vals = $stash->operate(['bar', 0, 'LIST', ['values']]);
ok( $vals );
match( join(', ', sort @$vals), '20, 30' );

$vals = $stash->operate(['bar', 0, 'LIST', ['each']]);
ok( $vals );
match( join(', ', sort @$vals), '20, 30, baz, boz' );

$result = $stash->operate(['obj', 0, 'LIST', 0]);
ok( $result );
match( $result->[0]->TT_ITEM(10), 'Foo->TT_ITEM(10)' );

match( $stash->operate(['objx', 0, 'LIST', [1]]),
       'Bar->TT_LIST(1)' );


#------------------------------------------------------------------------
# HASH
#------------------------------------------------------------------------

$result = $stash->operate(['foo', 0, 'HASH', 0]);
ok( $result );
match($result->{ item }, 10);

$result = $stash->operate(['bar', 0, 'HASH', 0]);
ok( $result );
match($result->{ baz }, 20);
match($result->{ boz }, 30);

$result = $stash->operate(['items', 0, 'HASH', 0]);
ok( $result );
match($result->{ 1 }, 2);
match( $result->{ 3 }, '' );

$result = $stash->operate(['obj', 0, 'HASH', 0]);
ok( $result );
match( $result->{ item }->TT_ITEM(10), 'Foo->TT_ITEM(10)' );

match( $stash->operate(['objx', 0, 'HASH', [1]]),
       'Bar->TT_HASH(1)' );


#------------------------------------------------------------------------
# LENGTH
#------------------------------------------------------------------------


my $length = $stash->operate(['foo', 0, 'LENGTH', 0]);
match($length, 2);

$length = $stash->operate(['bar', 0, 'LENGTH', 0]);
match( $length, 2 );

$length = $stash->operate(['baz', 0, 'LENGTH', 0]);
match( $length, 2 );

$length = $stash->operate(['obj', 0, 'LENGTH', 0]);
match( $length, 1 );

match( $stash->operate(['objx', 0, 'LENGTH', [1]]),
       'Bar->TT_LENGTH(1)' );


#------------------------------------------------------------------------
# SIZE
#------------------------------------------------------------------------

match( $stash->operate(['foo', 0, 'SIZE', 0]), 1 );
match( $stash->operate(['bar', 0, 'SIZE', 0]), 2 );
match( $stash->operate(['baz', 0, 'SIZE', 0]), 2 );
match( $stash->operate(['obj', 0, 'SIZE', 0]), 1 );
match( $stash->operate(['objx', 0, 'SIZE', [1]]), 'Bar->TT_SIZE(1)' );

#------------------------------------------------------------------------
# MAX
#------------------------------------------------------------------------

match( $stash->operate(['foo', 0, 'MAX', 0]), 0 );
match( $stash->operate(['bar', 0, 'MAX', 0]), 1 );
match( $stash->operate(['baz', 0, 'MAX', 0]), 1 );
match( $stash->operate(['obj', 0, 'MAX', 0]), 0 );
match( $stash->operate(['objx', 0, 'MAX', [1]]), 'Bar->TT_MAX(1)' );

#------------------------------------------------------------------------
# FIRST
#------------------------------------------------------------------------

match( $stash->operate(['foo', 0, 'FIRST', 0]), '10' );

$result = $stash->operate(['bar', 0, 'FIRST', 0]);
match( $result, 'boz' );
$result = $stash->operate(['baz', 0, 'FIRST', 0]);
match( $result, 'boz' );

$result = $stash->operate(['obj', 0, 'FIRST', 0]);
match( $result->plonk(), 'Plonk! []' );

match( $stash->operate(['objx', 0, 'FIRST', [3.14]]), 'Bar->TT_FIRST(3.14)' );

#------------------------------------------------------------------------
# LAST
#------------------------------------------------------------------------

match( $stash->operate(['foo', 0, 'LAST', 0]), '10' );

$result = $stash->operate(['bar', 0, 'LIST', ['hashrefs'], 'LAST', 0]);
match( $result->{ key }, 'boz' );
match( $result->{ value }, '30' );
$result = $stash->operate(['baz', 0, 'LIST', ['hashrefs'], 'LAST', 0]);
match( $result->{ key }, 'boz' );
match( $result->{ value }, '80' );

$result = $stash->operate(['obj', 0, 'LAST', 0]);
match( $result->plonk(), 'Plonk! []' );

match( $stash->operate(['objx', 0, 'LAST', [3.14]]), 'Bar->TT_LAST(3.14)' );


#------------------------------------------------------------------------
# CODEREF
#------------------------------------------------------------------------


my $code = $stash->operate(['foo', 0, 'CODEREF', 0]);
my $value = &$code();

$code = $stash->operate(['bar', 0, 'LENGTH', 0, 'CODEREF', 0]);
ok( $code );
$length = &$code();
match( $length, 2 );

$code = $stash->operate(['objx', 0, 'LENGTH', 0, 'CODEREF', [1,2]]);
ok( $code );
$result = &$code();
match( $result, 'Bar->TT_LENGTH(1, 2)' );
$result = &$code(3, 4);
match( $result, 'Bar->TT_LENGTH(1, 2, 3, 4)' );

$code = $stash->operate(['wiz', 0, 'CODEREF', [1,2]]);
ok( $code );
$result = &$code();
match( $result, 'wiz(1, 2)' );
$result = &$code(3, 4);
match( $result, 'wiz(1, 2, 3, 4)' );

$stash->{ plink } = 'Plonk!';
match( $stash->operate('plink'), 'Plonk!');
$code = $stash->operate(['plink', 0, 'CODEREF', 0]);
ok( $code );
match( &$code, 'Plonk!' );
$stash->{ plink } = 'Plunk!';
match( &$code, 'Plunk!' );

#------------------------------------------------------------------------
# ASSIGN
#------------------------------------------------------------------------

match( $stash->operate([ 'foo', 0, 'ASSIGN', 50 ]),
       '50' );
match( $stash->operate([ 'foo', 0 ]),
       '50' );

match( $stash->operate([ 'blat', 0, 'ASSIGN', 60 ]),
       '60' );
match( $stash->operate([ 'blat', 0 ]),
       '60' );

match( $stash->set(splat => 70), 70 );
match( $stash->get('splat'), 70 );

match( $stash->operate([ 'foo', [3, 2, 1], 'ASSIGN', 10 ]),
       3 );

match( $stash->operate([ 'foo', 0, 'wiz', 0, 'ASSIGN', 10 ]),
       '' );

match( $stash->operate([ LVALUE => 1, blim => 0, 
			  blam => 0,
			 ASSIGN => 99 ]),
       99 );

match( $stash->operate([ blim => 0, blam => 0 ]),
       99 );


#$Template::Stash::v3::DEBUG = 1;
#$Template::Stash::v3::WARNINGS = 1;

$stash->operate([ 'obj', [1, 2, 3], 'ASSIGN', 10 ]);
#$stash->operate([ 'objx', [1, 2, 3], 'ASSIGN', 10 ]);
match( $stash->operate([ 'objx', 0, 'plonk', [7], 'ASSIGN', 10 ]), 
       'Plonk! [7, 10]' );
match( $stash->operate([ 'objx', 0, 'plank', [8], 'ASSIGN', 10 ]), 
       'Bar->plank(8, 10)' );
match( $stash->operate([ 'objx', 0, 'plunk', 0, 'ASSIGN', 10 ]),
       'Bar->plunk(10)' );

match( $stash->operate([ LVALUE => 1, 'one', 0, 'two', 0, 
		         'three', 0, 
			 ASSIGN => 10 ]),
       '10' );


match( $stash->operate([ 'one', 0, 'two', 0, 'three', 0, ]),
       '10' );



#$Template::Stash::v3::DEBUG = 1;
#print STDERR '-' x 72, "\n";
match( $stash->operate([ 'one', 0, 'two', 0, 'three', 0,
			 DEFAULT => 20 ]),
       '10' );

match( $stash->operate([ LVALUE  => 1, one => 0, three => 0, five => 0,
			 DEFAULT => 7, ]),
       '7' );

match( $stash->operate([ one => 0, three => 0, five => 0 ]),
       '7' );


